using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using Phidgets;
using Phidgets.Events;
using System.Collections;

namespace WindowsApplication3
{
    public partial class Form1 : Form
    {
        Graphics g;                     // Graphics object for drawing the graph
        private MotorControl actuator;  // Phidgets MotorControl object for the motor controller board
        
        public Form1()
        {
            InitializeComponent();
        }
        
        // This function plots a point(x,y) of the specified color
        public void plotPoint(double x, double y, Color pointColor)
        {
            g.SmoothingMode = SmoothingMode.AntiAlias;
            Pen p = new Pen(pointColor, 2);
            g.DrawEllipse(p, (float)x, 250 - (float)y, 1, 1);
            p.Dispose();
        }

        // This function plots a line of the specified color
        public void plotLine(double x1, double y1, double x2, double y2, Color pointColor)
        {
            g.SmoothingMode = SmoothingMode.AntiAlias;
            Pen p = new Pen(pointColor, 2);
            g.DrawLine(p, (float)x1, 250 - (float)y1, (float)x2, 250 - (float)y2);
            p.Dispose();
        }

        // This function draws the lines needed to refresh the graph when it loops
        public void wipeGraph(double x)
        {
            Pen p = new Pen(Color.White, 1);
            Pen p2 = new Pen(Color.Gray, 1);
            // Draw the black line
            g.DrawLine(p, (float)x, 0, (float)x, 257);
            // Clean the area immediately after with a gray line
            g.DrawLine(p2, (float)(x + 0.5), 1, (float)(x + 0.5), 257);
            p.Dispose();
            p2.Dispose();
        }

        // This function clears the graph entirely
        public void clearGraph()
        {
            g = pictureBox1.CreateGraphics();
            g.Clear(Color.LightGray);
        }

        // Detach event handler
        void actuator_Detach(object sender, DetachEventArgs e)
        {
            // Clear motor controller fields
            attachedTxt.Text = "Not Attached";
            nameTxt.Text = "- - -";
            serialTxt.Text = "- - -";
            versionTxt.Text = "- - -";
        }

        // Attach event handler
        void actuator_Attach(object sender, AttachEventArgs e)
        {
            // Populate motor controller fields
            MotorControl attached = (MotorControl)sender;
            attachedTxt.Text = attached.Attached.ToString();
            nameTxt.Text = attached.Name;
            serialTxt.Text = attached.SerialNumber.ToString();
            versionTxt.Text = attached.Version.ToString();
            attached.motors[0].Acceleration = attached.motors[0].AccelerationMax;
            attached.Ratiometric = true;

            // Populate motor selection combo box
            int i;
            for (i = 0; i < attached.motors.Count; i++)
            {
                motorCmb.Items.Add(i);
            }

            // Populate feedback sensor combo box
            for (i = 0; i < attached.sensors.Count; i++)
            {
                analogCmb.Items.Add(i);
            }

            // Default to index 0 for both
            motorCmb.SelectedIndex = 0;
            analogCmb.SelectedIndex = 0;
        }


        //Variables
        double Kp = 1;                 // Kp is the control gain that scales the proportional term of the control loop
        double Ki = 1;                 // Ki is the control gain that scales the integral term of the control loop
        double Kv = 0.25;              // Kv is the control gain that scales the target velocity
        double Kf = 1;                 // Kf is the control gain that scales the feed forward term
        double output;                 // output is the result from the control loop calculation (a.k.a the duty cycle sent to the motor controller)
        double velocityError;          // velocityError is the difference between the target and the actual velocity
        double positionError;          // positionError is the difference between the target and the actual position
        double velocityTarget;         // velocityTarget is the target velocity based on the difference between the current and target positions
        double velocityCurrent;        // velocityCurrent is an estimate of the actuator's velocity based on its current position and its position 64 samples ago
        double positionCurrent;        // positionCurrent is the current position of the actuator according to the feedback potentiometer
        double positionTarget;         // positionTarget is the initial target position chosen using the position slider
        double positionOld = 0;        // positionOld is the position of the actuator 64 samples ago (for the purpose of calculating velocity)
        double integral;               // integral is the integral term of the control loop
        double dt;                     // dt is the amount of time between iterations of the control loop
        double maxOutput = 100;        // maxOutput is the selected maximum velocity for the control loop (this limit stays constant even with changing load)
        double deadBand = 0;           // deadBand is the amount of error that can still be present when the motor stops
        bool started = false;          // started controls whether or not the control loop is active
        double graphTimeCount = 0;     // graphTimeCount is the X position variable for the graph

        double velocityTargetLast;
        double velocityCurrentLast;
        double dutyCycleLast;

        // positionArray is used to store the last 64 position values for the purpose of estimating velocity
        double[] positionArray = new double[64];

        // FFarray is the array of expected motor velocities (units: sensorValue per second) that correspond with the duty cycle in increments of 20
        double[] FFarray = new double[11] {-228, -182, -136, -91, -46, 0, 46, 91, 136, 182, 228};

        // FFdutycycle is the array of duty cycles in increments of 20. This array is constant.
        int[] FFdutycycle = new int[11] {-100,-80, -60,- 40,-20, 0,20, 40, 60, 80,100};


        // This function interpolates between two points
        static double interpolate(double x0, double y0, double x1, double y1, double x)
        {
            return y0 * (x - x1) / (x0 - x1) + y1 * (x - x0) / (x1 - x0);
        }

        //returns Kf, a, integer gain value between -100 and 100.
        //recieves feedforward, a speed measured in sensorvalue per second
        int getFF(double feedforward)
        {
            for (int i = 0; i < 10; i++)
            {
                // If feedforward is between two points on the characteristic curve,
                if((feedforward >= FFarray[i]) && (feedforward <= FFarray[i+1]))
                {
                    return (int)interpolate(FFarray[i], FFdutycycle[i], FFarray[i + 1], FFdutycycle[i + 1], feedforward);
                }
            }

            // If feedforward is greater than the maximum value of the characteristic curve,
            if (feedforward > FFarray[10])
                return 100; // return the maximum duty cycle
            // If feedforward is less than the minimum value of the characteristic curve,
            else if (feedforward < FFarray[0])
                return -100; // return the minimum duty cycle
            // Else, something is wrong with the characteristic curve and the motor should be stopped
            else
                return 0;
        }

        // This function does the control system calculations and sets output to the duty cycle that the motor needs to run at.
        void ControlLoop()
        {
            // Update position array and variables
            for (int i = 63; i > 0; i--)
            {
                positionArray[i] = positionArray[i-1];
            }
            positionArray[0] = positionCurrent;
            positionOld = positionArray[63];

            // Update velocity variables
            velocityCurrent = (positionCurrent - positionOld) / (64 * dt);
            velocityTarget = (positionTarget - positionCurrent) * Kv;
          
            // Limit target velocity to the user-specified amount
            if (velocityTarget >= maxOutput)
                velocityTarget = maxOutput;
            if (velocityTarget <= -maxOutput)
                velocityTarget = -maxOutput;
     
            // Update error values
            velocityError = velocityTarget - velocityCurrent;
            positionError = positionCurrent - positionTarget;

            // If the error is within the specified deadband, and the motor is moving slowly enough,
            // Or if the motor's target is a physical limit and that limit is hit (within deadband margins),
            if ((Math.Abs(positionError) <= deadBand && Math.Abs(output) < 5)
                || (positionTarget < (deadBand + 1) && positionCurrent < (deadBand + 1))
                || (positionTarget > (999 - deadBand) && positionCurrent > (999 - deadBand)))
            {
                // Stop the motor
                output = 0;
                positionError = 0;
            }
            // Else, update motor duty cycle with the newest output value
            else
            {
                // This equation represents the control loop
                // The first term is the velocity error multiplied by the proportional constant,
                // The second term is the feed-forward velocity estimate,
                // and the third term is the accumulated integral multiplied by the integral constant.
                output = (Kp * velocityError) + (Kf * getFF(velocityTarget)) + (Ki * integral);

                // Saturate the output at +/- 100 duty cycle,
                if (output > 100)
                    output = 100;
                else if (output < -100)
                    output = -100;
                // else, accumulate the integral
                else
                    integral += (velocityError * dt);
            }

            //Update textbox values
            outputTxt.Text = Math.Round(velocityCurrent,2).ToString();
            targetvelocityTxt.Text = Math.Round(velocityTarget, 2).ToString();
            errorTxt.Text = positionError.ToString();
            velocityErrorTxt.Text = (velocityTarget - velocityCurrent).ToString();

            // If the time counter of the graph exceeds the width of the frame, loop back to the beginning
            if (graphTimeCount > pictureBox1.Size.Width)
                graphTimeCount = 0;
            
            // Draw the wiper line for the graph
            wipeGraph(graphTimeCount);

            // Plot the points each graphed variable (if enabled)
            if(currentPositionCheck.Checked)
                plotPoint(graphTimeCount, positionCurrent / 4, Color.Red);
            if(dutyCycleCheck.Checked)
            {
                plotPoint(graphTimeCount, Math.Abs(output) * 2.5, Color.Black);
                plotLine(graphTimeCount-0.5, dutyCycleLast, graphTimeCount, Math.Abs(output) * 2.5, Color.Black);
            }
            if (actualVelocityCheck.Checked)
            {
                plotPoint(graphTimeCount, Math.Abs(velocityCurrent), Color.Blue);
                plotLine(graphTimeCount - 0.5, velocityCurrentLast, graphTimeCount, Math.Abs(velocityCurrent), Color.Blue);
            }
            if(targetVelocityCheck.Checked)
            {
                plotPoint(graphTimeCount, Math.Abs(velocityTarget), Color.Purple);
                plotLine(graphTimeCount - 0.5, velocityTargetLast, graphTimeCount, Math.Abs(velocityTarget), Color.Purple);
            }
            if (deadBandCheck.Checked)
            {
                plotPoint(graphTimeCount, (positionTarget + deadBand) / 4, Color.Yellow);
                plotPoint(graphTimeCount, (positionTarget - deadBand) / 4, Color.Yellow);
            }
            if(targetPositionCheck.Checked)
                plotPoint(graphTimeCount, positionTarget / 4, Color.Green);

            dutyCycleLast = Math.Abs(output) * 2.5;
            velocityCurrentLast = Math.Abs(velocityCurrent);
            velocityTargetLast = Math.Abs(velocityTarget);

            // Increment the graph time counter
            graphTimeCount += 0.5;

        }

        // This function gets the target position from the slider and starts the PID control loop
        private void startButton_Click(object sender, EventArgs e)
        {
            //Initialize graph and flags
            clearGraph();
            graphTimeCount = 0;
            started = true;

            //Draw Axis label for graph
            Graphics h = pictureBox2.CreateGraphics();
            Pen p = new Pen(Color.Black, 1);
            Font drawFont = new Font("Microsoft Sans Serif", (float)9);
            SolidBrush drawBrush = new SolidBrush(Color.Black);
            StringFormat drawFormat = new StringFormat();
            drawFormat.FormatFlags = StringFormatFlags.DirectionVertical;
            h.DrawString("Position", drawFont, drawBrush, 6, 100, drawFormat);
            h.DrawLine(p, 20, 2, 27, 2);
            h.DrawLine(p, 20, 252, 27, 252);

            //Get the target position and display it in the text box
            targetTxt.Text = PositionTrk.Value.ToString();

            try
            {
                // Get the control gains
                Kp = Convert.ToDouble(kpTxt.Text);
                Kv = Convert.ToDouble(kvTxt.Text);
                Ki = Convert.ToDouble(kiTxt.Text);
                Kf = Convert.ToDouble(kfTxt.Value);

                //Initialize the position array so the control system starts with a correct current velocity
                for (int i = 0; i < 63; i++)
                {
                    positionArray[i] = positionCurrent;
                }

                // Get the motor variables
                maxOutput = VelocityTrk.Value;
                deadBand = Convert.ToDouble(deadTxt.Text);
                positionTarget = PositionTrk.Value;
            }
            catch
            {
                //Do nothing if any input value is not valid
                return;
            }
        }

        // This is the event handler for the attached feedback sensor
        void actuator_SensorUpdate(object sender, SensorUpdateEventArgs e)
        {
            // If the selected sensor input is the one that triggered this handler,
            if ((e.Index == 0 && analogCmb.Text == "0") || (e.Index == 1 && analogCmb.Text == "1"))
            {
                //Update rate determines the time interval
                dt = 0.008;

                //Update actual position text box
                feedbackTxt.Text = e.Value.ToString();

                if (targetTxt.Text != "" && feedbackTxt.Text != "")
                {
                    errorTxt.Text = Convert.ToString(Convert.ToInt32(targetTxt.Text) - Convert.ToInt32(feedbackTxt.Text));
                }

                positionCurrent = e.Value;

                //Run the Velocity Control loop
                if (started)
                {
                    ControlLoop();
                }
                //Update the motor controller with the new duty cycle
                try
                {
                    actuator.motors[Convert.ToInt32(motorCmb.Text)].Velocity = Math.Round(output);
                }
                catch
                {
                    //System.Windows.Forms.MessageBox.Show("Invalid output:" + Math.Round(output));
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Set up motor object and event handlers
            actuator = new MotorControl();
            actuator.Attach += new AttachEventHandler(actuator_Attach);
            actuator.Detach += new DetachEventHandler(actuator_Detach);
            actuator.SensorUpdate += new SensorUpdateEventHandler(actuator_SensorUpdate);

            // Try to open the motor controller
            // If this fails, the user can retry by reconnecting the USB cable, triggering an attach event
            try
            {
                actuator.open();
                attachedTxt.Text = "Waiting for Attach...";
                actuator.waitForAttachment(10000);
            }
            catch
            {
                attachedTxt.Text = "Attach Timed Out.";
            }

        }

        // These functions update the percentage labels when the sliders are moved
        private void PositionTrk_Scroll(object sender, EventArgs e)
        {
            // Move position percentage label
            PositionTxt.Text = (PositionTrk.Value / 10).ToString() + "%";
            PositionTxt.Location = new Point(Convert.ToInt32(Math.Round(PositionTrk.Value * 0.675)) + 18, 203);
        }

        private void VelocityTrk_Scroll(object sender, EventArgs e)
        {
            // Move velocity percentage label
            velocityTrkTxt.Text = (VelocityTrk.Value).ToString() + " SV/s";
            velocityTrkTxt.Location = new Point(Convert.ToInt32(Math.Round(VelocityTrk.Value * 0.796)) + 18, 139);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            
            actuator.Attach -= actuator_Attach;
            actuator.Detach -= actuator_Detach;
            actuator.SensorUpdate -= actuator_SensorUpdate;

            started = false;
 
            if (actuator.Attached)
            {
                output = 0;
                actuator.motors[Convert.ToInt32(motorCmb.Text)].Velocity = 0;
                Application.DoEvents();
                actuator.close();
                actuator = null;
            }
            
        }

        private void trackBar100_Scroll(object sender, EventArgs e)
        {
            if (trackBar100.Value < trackBar80.Value)
                trackBar100.Value = trackBar80.Value;
            textBox100.Text = trackBar100.Value.ToString();
            updateCurve();
        }

        private void trackBar80_Scroll(object sender, EventArgs e)
        {
            if (trackBar80.Value > trackBar100.Value)
                trackBar80.Value = trackBar100.Value;
            if (trackBar80.Value < trackBar60.Value)
                trackBar80.Value = trackBar60.Value;
            textBox80.Text = trackBar80.Value.ToString();
            updateCurve();
        }

        private void trackBar60_Scroll(object sender, EventArgs e)
        {
            if (trackBar60.Value > trackBar80.Value)
                trackBar60.Value = trackBar80.Value;
            if (trackBar60.Value < trackBar40.Value)
                trackBar60.Value = trackBar40.Value;
            textBoxR60.Text = (-trackBarR60.Value).ToString();
            textBox60.Text = trackBar60.Value.ToString();
            updateCurve();
        }

        private void trackBar40_Scroll(object sender, EventArgs e)
        {
            if (trackBar40.Value > trackBar60.Value)
                trackBar40.Value = trackBar60.Value;
            if (trackBar40.Value < trackBar20.Value)
                trackBar40.Value = trackBar20.Value;
            textBox40.Text = trackBar40.Value.ToString();
            updateCurve();
        }

        private void trackBar20_Scroll(object sender, EventArgs e)
        {
            if (trackBar20.Value > trackBar40.Value)
                trackBar20.Value = trackBar40.Value;
            textBox20.Text = trackBar20.Value.ToString();
            updateCurve();
        }

        private void trackBarR20_Scroll(object sender, EventArgs e)
        {
            if (trackBarR20.Value > trackBarR40.Value)
                trackBarR20.Value = trackBarR40.Value;
            textBoxR20.Text = (-trackBarR20.Value).ToString();
            updateCurve();
        }

        private void trackBarR40_Scroll(object sender, EventArgs e)
        {
            if (trackBarR40.Value > trackBarR60.Value)
                trackBarR40.Value = trackBarR60.Value;
            if (trackBarR40.Value < trackBarR20.Value)
                trackBarR40.Value = trackBarR20.Value;
            textBoxR40.Text = (-trackBarR40.Value).ToString();
            updateCurve();
        }

        private void trackBarR60_Scroll(object sender, EventArgs e)
        {
            if (trackBarR60.Value > trackBarR80.Value)
                trackBarR60.Value = trackBarR80.Value;
            if (trackBarR60.Value < trackBarR40.Value)
                trackBarR60.Value = trackBarR40.Value;
            textBoxR60.Text = (-trackBarR60.Value).ToString();
            updateCurve();
        }

        private void trackBarR80_Scroll(object sender, EventArgs e)
        {
            if (trackBarR80.Value > trackBarR100.Value)
                trackBarR80.Value = trackBarR100.Value;
            if (trackBarR80.Value < trackBarR60.Value)
                trackBarR80.Value = trackBarR60.Value;
            textBoxR80.Text = (-trackBarR80.Value).ToString();
            updateCurve();
        }

        private void trackBarR100_Scroll(object sender, EventArgs e)
        {
            if (trackBarR100.Value < trackBarR80.Value)
                trackBarR100.Value = trackBarR80.Value;
            textBoxR100.Text = (-trackBarR100.Value).ToString();
            updateCurve();
        }

        private void setCurveButton_Click(object sender, EventArgs e)
        {
            double topspeed = Convert.ToDouble(topSpeedTxt.Value);
            double stroke = Convert.ToDouble(strokeLengthTxt.Value);

            double svps = (topspeed * 1000) / stroke;

            try
            {
                trackBarR100.Value = Convert.ToInt32(svps);
                textBoxR100.Text = (Math.Round(-svps).ToString());

                trackBarR80.Value = Convert.ToInt32(svps * 0.8);
                textBoxR80.Text = (Math.Round(svps * -0.8).ToString());

                trackBarR60.Value = Convert.ToInt32(svps * 0.6);
                textBoxR60.Text = (Math.Round(svps * -0.6).ToString());

                trackBarR40.Value = Convert.ToInt32(svps * 0.4);
                textBoxR40.Text = (Math.Round(svps * -0.4).ToString());

                trackBarR20.Value = Convert.ToInt32(svps * 0.2);
                textBoxR20.Text = (Math.Round(svps * -0.2).ToString());

                trackBar20.Value = Convert.ToInt32(svps * 0.2);
                textBox20.Text = (Math.Round(svps * 0.2).ToString());

                trackBar40.Value = Convert.ToInt32(svps * 0.4);
                textBox40.Text = (Math.Round(svps * 0.4).ToString());

                trackBar60.Value = Convert.ToInt32(svps * 0.6);
                textBox60.Text = (Math.Round(svps * 0.6).ToString());

                trackBar80.Value = Convert.ToInt32(svps * 0.8);
                textBox80.Text = (Math.Round(svps * 0.8).ToString());

                trackBar100.Value = Convert.ToInt32(svps);
                textBox100.Text = (Math.Round(svps).ToString());

                updateCurve();
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("The values you have chosen are out of the expected range for this program.", "Error: Out of Range");
            }
        }

        private void updateCurve()
        {
            Graphics g2 = curveBox.CreateGraphics();

            Pen p = new Pen(Color.Black, 1);
            Pen p2 = new Pen(Color.Red, 2);
            Pen p3 = new Pen(Color.Gray, 1);

            g2.Clear(Color.White);

            g2.DrawLine(p3, 30, 0, 30, 300);
            g2.DrawLine(p3, 60, 0, 60, 300);
            g2.DrawLine(p3, 90, 0, 90, 300);
            g2.DrawLine(p3, 120, 0, 120, 300);
            g2.DrawLine(p3, 180, 0, 180, 300);
            g2.DrawLine(p3, 210, 0, 210, 300);
            g2.DrawLine(p3, 240, 0, 240, 300);
            g2.DrawLine(p3, 270, 0, 270, 300);

            g2.DrawLine(p, 150, 0, 150, 300);
            g2.DrawLine(p, 0, 150, 300, 150);

            g2.DrawLine(p2, 300, (float)(150 - (trackBar100.Value * 0.3)), 270, (float)(150 - (trackBar80.Value * 0.3)));
            g2.DrawLine(p2, 270, (float)(150 - (trackBar80.Value * 0.3)), 240, (float)(150 - (trackBar60.Value * 0.3)));
            g2.DrawLine(p2, 240, (float)(150 - (trackBar60.Value * 0.3)), 210, (float)(150 - (trackBar40.Value * 0.3)));
            g2.DrawLine(p2, 210, (float)(150 - (trackBar40.Value * 0.3)), 180, (float)(150 - (trackBar20.Value * 0.3)));
            g2.DrawLine(p2, 180, (float)(150 - (trackBar20.Value * 0.3)), 150, 150);
            g2.DrawLine(p2, 150, 150, 120, (float)(150 + (trackBarR20.Value * 0.3)));
            g2.DrawLine(p2, 120, (float)(150 + (trackBarR20.Value * 0.3)), 90, (float)(150 + (trackBarR40.Value * 0.3)));
            g2.DrawLine(p2, 90, (float)(150 + (trackBarR40.Value * 0.3)), 60, (float)(150 + (trackBarR60.Value * 0.3)));
            g2.DrawLine(p2, 60, (float)(150 + (trackBarR60.Value * 0.3)), 30, (float)(150 + (trackBarR80.Value * 0.3)));
            g2.DrawLine(p2, 30, (float)(150 + (trackBarR80.Value * 0.3)), 0, (float)(150 + (trackBarR100.Value * 0.3)));

            FFarray = new double[] { -trackBarR100.Value, -trackBarR80.Value, -trackBarR60.Value, -trackBarR40.Value, -trackBarR20.Value, 0, trackBar20.Value, trackBar40.Value, trackBar60.Value, trackBar80.Value, trackBar100.Value };
        }


        //////////////////////////////////////
        // Help functions beyond this point //
        //////////////////////////////////////

        private void targetpositionLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the position specified by the user for the actuator to move towards. Position is measured from 0 (completely retracted) to 1000 (fully extended).","Help: Target Position");
        }

        private void actualpositionLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the actual position of the actuator according to the feedback potentiometer connected to the analog input. Position is measured from 0 (completely retracted) to 1000 (fully extended).", "Help: Actual Position");
        }

        private void errorLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the difference between the target position and the actual position. The control loop will stop when position error is within the deadband value.", "Help: Position Error");
        }

        private void velocityErrorLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the difference between the target velocity and the estimated velocity. The goal of this control loop is to reduce absolute value of velocity error while it is running.", "Help: Velocity Error");
        }

        private void velocityLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the velocity in SensorValue per second that the control loop will try to maintain. This value can be limited with the velocity slider.", "Help: Output Velocity");
        }

        private void motorLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the index of the motor being controlled. On a controller with only one motor, this will always be zero.", "Help: Motor Index");
        }

        private void analogLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the index of the analog input connected to the feedback potentiometer. Only change events from the selected index will be used.", "Help: Sensor Index");
        }

        private void kpLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the proportional gain of the control loop. Increasing Kp while keeping other gains constant should result in the motor reaching the target faster, but may also increase the amount it overshoots the target by.", "Help: Kp");
        }

        private void kiLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the integral gain of the control loop. Increasing Ki while keeping other gains constant should result in the motor reaching the target faster and more accurately, but may also increase the amount it overshoots the target by and increase the time it takes to settle on the exact target.", "Help: Ki");
        }

        private void deadBandLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the amount of acceptable error on either side of the target. For example, a deadband of 5 and a target position of 450 would cause the motor to stop at any position between 445 and 455 inclusive.", "Help: DeadBand");
        }

        private void attachedLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This shows whether or not a Phidget Motor Controller has attached. If you're having trouble with yout device attaching, ensure that it is the specific product that this example is designed for, and try disconnecting and reconnecting the USB cable.", "Help: Attached");
        }

        private void nameLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the name of the attached device being used to control the motor.", "Help: Device Name");
        }

        private void serialLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the serial number of the device being used to control the motor.", "Help: Serial Number");
        }

        private void versionLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the firmware version of the device being used to control the motor.", "Help: Version");
        }

        private void velocitysliderLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the velocity limit of the actuator in units of SensorValue per second. The control loop will try to limit the target velocity to this value, even with a changing load.", "Help: Velocity Slider");
        }

        private void targetsliderLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the target position of the actuator on a scale of 0 to 100% of the full stroke length. Select a position using this slider and press the start button to begin the control loop.", "Help: Target Position Slider");
        }

        private void startButton_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("Press this button to start the control loop. If the actuator isn't moving, make sure the maximum velocity is set to a nonzero value, check your control gains, and make sure the actuator's controller is the device that is attached to this program.", "Help: Start Button");
        }

        private void actuatorImage_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is a linear actuator. This program will not be of much use to you if you don't have one of these.", "Help: Pictures");
        }

        private void pictureBox1_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the graphing area. The graph will show the target position, velocity, duty cycle, deadbands, and current position in a scroll-refresh format.", "Help: Graph");
        }

        private void kvLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the velocity gain of the control loop. This is the amount that the target velocity is scaled by. This value matters when the motor is nearing its target position. A lower Kv will result in a more gradual deceleration towards the target, while a higher Kv will move more directly to the target. If Kv is too high, the motor will oscillate around the target position.", "Help: Kv");
        }

        private void estVelocityLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the estimated velocity of the actuator based on the difference between its current position and its position 64 samples ago.", "Help: Estimated Velocity");
        }

        private void dutycycleLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the actual duty cycle being sent to the motor. Duty cycle ranges from 100% to -100%.", "Help: Duty Cycle");
        }

        private void kfLabel_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This is the feed-forward gain of the control loop. This value scales the amount that the user-defined characteristic curve affects the control loop.", "Help: Kf");
        }

        private void trackBar100_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("These sliders control the characteristic curve for your servo motor. The more accurate this curve is, the better the feed-forward term of the control loop will function.", "Help: Characteristic Curve");
        }

        private void curveBox_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("This graph plots the characteristic curve for your servo motor. The more accurate this curve is, the better the feed-forward term of the control loop will function.", "Help: Characteristic Curve");
        }

        private void topSpeedTxt_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("Put the maximum speed of your actuator here. This will be used to estimate the characteristic curve.", "Help: Top Speed");
        }

        private void strokeLengthTxt_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("Put the stroke length of your actuator here. This will be used to estimate the characteristic curve.", "Help: Stroke Length");
        }

        private void setCurveButton_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            System.Windows.Forms.MessageBox.Show("Press this button after filling in the maximum speed and stroke length of your actuator to automatically estimate the characteristic curve.", "Help: Characteristic Curve");
        }
    }
}